@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2014	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -101,6 +101,13 @@ typedef enum {
 	POOL_DEADLOCK
 } POOL_STATUS;
 
+typedef enum {
+	POOL_SOCKET_CLOSED = 0,
+	POOL_SOCKET_VALID,
+	POOL_SOCKET_ERROR,
+	POOL_SOCKET_EOF
+} POOL_SOCKET_STATE;
+
 /* protocol major version numbers */
 #define PROTO_MAJOR_V2	2
 #define PROTO_MAJOR_V3	3
@@ -216,7 +223,7 @@ typedef struct {
 	char kind;	/* kind cache */
 
 	/* true if remote end closed the connection */
-	bool EOF_on_socket;
+	POOL_SOCKET_STATE socket_state;
 	/*
 	 * frontend info needed for hba
 	 */
@@ -695,4 +702,6 @@ extern int connect_unix_domain_socket_by_port(int port, char *socket_dir, bool r
 extern int pool_pool_index(void);
 
 extern int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps);
+extern int write_status_file(void);
+
 #endif /* POOL_H */
