@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2015	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -87,7 +87,7 @@ static POOL_CONNECTION *get_connection(int front_end_fd, SockAddr *saddr);
 static POOL_CONNECTION_POOL *get_backend_connection(POOL_CONNECTION *frontend);
 static StartupPacket *StartupPacketCopy(StartupPacket *sp);
 static void print_process_status(char *remote_host,char* remote_port);
-static bool backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volatile backend);
+static bool backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volatile backend, bool frontend_invalid);
 static void free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT *cp);
 static int choose_db_node_id(char *str);
 static void child_will_go_down(int code, Datum arg);
@@ -211,11 +211,22 @@ void do_child(int *fds)
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
 		POOL_PROCESS_CONTEXT *session;
+		bool frontend_invalid = getfrontendinvalid();
+
 		disable_authentication_timeout();
 		/* Since not using PG_TRY, must reset error stack by hand */
 		error_context_stack = NULL;
 
-		EmitErrorReport();
+		/*
+		 * Do not emit an error when EOF was encountered on frontend connection before the session
+		 * was initialized. This is the normal behavior of psql to close and reconnect
+		 * the connection when some authentication method is used
+		 */
+		if(pool_get_session_context(true) ||
+		   !child_frontend ||
+		   child_frontend->socket_state != POOL_SOCKET_EOF)
+			EmitErrorReport();
+
         /* process the cleanup in ProcessLoopContext which will get reset
          * during the next loop iteration
          */
@@ -224,7 +235,7 @@ void do_child(int *fds)
 		if (accepted)
 			connection_count_down();
 
-		backend_cleanup(&child_frontend, backend);
+		backend_cleanup(&child_frontend, backend, frontend_invalid);
 
 		session = pool_get_process_context();
 
@@ -275,6 +286,7 @@ void do_child(int *fds)
 		MemoryContextSwitchTo(ProcessLoopContext);
 		MemoryContextResetAndDeleteChildren(ProcessLoopContext);
 
+		backend = NULL;
 		idle = 1;
 
 		/* pgpool stop request already sent? */
@@ -362,7 +374,7 @@ void do_child(int *fds)
 			status = pool_process_query(child_frontend, backend, 0);
             if(status != POOL_CONTINUE)
             {
-                backend_cleanup(&child_frontend, backend);
+                backend_cleanup(&child_frontend, backend, 0);
                 break;
             }
 		}
@@ -405,7 +417,7 @@ void do_child(int *fds)
  * return true if backend connection is cached
  */
 static bool
-backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volatile backend)
+backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volatile backend, bool frontend_invalid)
 {
     StartupPacket *sp;
     bool cache_connection = false;
@@ -414,13 +426,13 @@ backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volat
         return false;
 
     sp = MASTER_CONNECTION(backend)->sp;
-    
+
     /*
-     * cach connection if connection is not for
+     * cach connection if frontend is not invalid, connection is not for
      * system db and connection cache configuration
      * parameter is enabled
      */
-    if (sp && pool_config->connection_cache != 0 && sp->system_db == false)
+    if (sp && pool_config->connection_cache != 0 && sp->system_db == false  && frontend_invalid == false )
     {
         if (*frontend)
         {
@@ -443,6 +455,22 @@ backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volat
         }
     }
 
+	if (cache_connection)
+	{
+		/*
+		 * For those special databases, and when frontend client exits abnormally,
+		 * we don't cache connection to backend.
+		 */
+		if ((sp &&
+			(!strcmp(sp->database, "template0") ||
+			 !strcmp(sp->database, "template1") ||
+			 !strcmp(sp->database, "postgres") ||
+			 !strcmp(sp->database, "regression"))) ||
+			 (*frontend != NULL &&
+			  ((*frontend)->socket_state == POOL_SOCKET_EOF ||
+			  (*frontend)->socket_state == POOL_SOCKET_ERROR)))
+			cache_connection = false;
+	}
 	/*
 	 * Close frontend connection
 	 */
@@ -450,16 +478,6 @@ backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volat
 	pool_close(*frontend);
 	*frontend = NULL;
 
-	/*
-	 * For those special databases we don't cache connection to backend.
-	 */
-    if (sp &&
-		(!strcmp(sp->database, "template0") ||
-		 !strcmp(sp->database, "template1") ||
-		 !strcmp(sp->database, "postgres") ||
-		 !strcmp(sp->database, "regression")))
-        cache_connection = false;
-
     if (cache_connection == false)
     {
 		pool_send_frontend_exits(backend);
@@ -2032,7 +2050,7 @@ static void get_backends_status(unsigned int *valid_backends, unsigned int *down
 	{
 		if (BACKEND_INFO(i).backend_status == CON_DOWN)
 			(*down_backends)++;
-		else if(VALID_BACKEND(i))
+		if (VALID_BACKEND(i))
 			(*valid_backends)++;
 	}
 }
@@ -2078,7 +2096,7 @@ static void validate_backend_connectivity(int front_end_fd)
 		{
 			fatal_error = true;
 			error_msg = "pgpool is not accepting any new connections";
-			error_detail = "all backend nodes are down, pgpool requires atleast one valid node";
+			error_detail = "all backend nodes are down, pgpool requires at least one valid node";
 			error_hint = "repair the backend nodes and restart pgpool";
 		}
 	}
@@ -2158,7 +2176,8 @@ get_connection(int front_end_fd, SockAddr *saddr)
 
     
 	/* set NODELAY and KEEPALIVE options if INET connection */
-	if (saddr->addr.ss_family == AF_INET)
+	if (saddr->addr.ss_family == AF_INET ||
+		saddr->addr.ss_family == AF_INET6)
 	{
 		int on = 1;
 
@@ -2371,6 +2390,8 @@ int send_to_pg_frontend(char* data, int len, bool flush)
 	int ret;
 	if (processType != PT_CHILD || child_frontend == NULL)
 		return -1;
+	if (child_frontend->socket_state != POOL_SOCKET_VALID)
+		return -1;
 	ret = pool_write_noerror(child_frontend, data, len);
 	if(flush && !ret)
 		ret = pool_flush_it(child_frontend);
@@ -2381,6 +2402,8 @@ int set_pg_frontend_blocking(bool blocking)
 {
 	if (processType != PT_CHILD || child_frontend == NULL)
 		return -1;
+	if (child_frontend->socket_state != POOL_SOCKET_VALID)
+		return -1;
 	if (blocking)
 		pool_unset_nonblock(child_frontend->fd);
 	else
